#include <algorithm>
#include <atomic>
#include <chrono>
#include <iostream>
#include <random>
#include <vector>

using namespace std::chrono;

void find_max_cas(std::vector<int> & vector) {
    std::atomic_int atomic_max_index;
#pragma omp parallel for num_threads(1) shared(vector, atomic_max_index)
    for (int i=0; i<vector.size(); ++i) {
        int tmp_index = atomic_max_index.load();
        while ((vector[i] > vector[tmp_index] || (vector[i] == vector[tmp_index] && i > tmp_index)) && 	 !atomic_max_index.compare_exchange_strong(tmp_index,i)) {
            tmp_index = atomic_max_index.load();
        }
    }
}

int main() {
    const int N = 1000000;
    std::vector<int> v(N);
    std::mt19937 rng;
    rng.seed(std::random_device()());
    std::uniform_int_distribution<int> dist(0, 255);
    std::generate(begin(v), end(v), [&]() { return dist(rng); });
    auto start = high_resolution_clock::now();
    find_max_cas(v);
    auto finish = high_resolution_clock::now();
    auto duration = duration_cast<milliseconds>(finish - start);
    std::cout << "\nElapsed time = " << duration.count() << " ms\n";
    return 0;
}
